@@ -9,9 +9,11 @@
 #include <glib.h>
 #include <libsoup/soup-message.h>
 
+G_BEGIN_DECLS
+
 /* HTTP Header Parsing */
 
-gboolean    soup_headers_parse_request      (char             *str, 
+gboolean    soup_headers_parse_request      (const char       *str, 
 					     int               len, 
 					     GHashTable       *dest, 
 					     char            **req_method,
@@ -23,7 +25,7 @@ gboolean    soup_headers_parse_status_line  (const char        *status_line,
 					     guint            *status_code,
 					     char            **reason_phrase);
 
-gboolean    soup_headers_parse_response     (char             *str, 
+gboolean    soup_headers_parse_response     (const char       *str, 
 					     int               len, 
 					     GHashTable       *dest,
 					     SoupHttpVersion  *ver,
@@ -41,4 +43,6 @@ char       *soup_header_param_copy_token    (GHashTable       *tokens,
 
 void        soup_header_param_destroy_hash  (GHashTable       *table);
 
+G_END_DECLS
+
 #endif /*SOUP_HEADERS_H*/
