@@ -70,6 +70,11 @@ finalize (GObject *object)
 	if (priv->uri)
 		soup_uri_free (priv->uri);
 
+	if (priv->auth)
+		g_object_unref (priv->auth);
+	if (priv->proxy_auth)
+		g_object_unref (priv->proxy_auth);
+
 	if (msg->request.owner == SOUP_BUFFER_SYSTEM_OWNED)
 		g_free (msg->request.body);
 	if (msg->response.owner == SOUP_BUFFER_SYSTEM_OWNED)
@@ -146,7 +151,6 @@ soup_message_class_init (SoupMessageClass *message_class)
 	 * @msg: the message
 	 *
 	 * Emitted immediately after writing a body chunk for a message.
-	 * (This is
 	 **/
 	signals[WROTE_CHUNK] =
 		g_signal_new ("wrote_chunk",
@@ -631,7 +635,7 @@ soup_message_add_header (GHashTable *hash, const char *name, const char *value)
 	old_value = g_hash_table_lookup (hash, name);
 
 	if (old_value)
-		g_slist_append (old_value, g_strdup (value));
+		old_value = g_slist_append (old_value, g_strdup (value));
 	else {
 		g_hash_table_insert (hash, g_strdup (name),
 				     g_slist_append (NULL, g_strdup (value)));
@@ -724,6 +728,112 @@ soup_message_foreach_header (GHashTable *hash, GHFunc func, gpointer user_data)
 	g_hash_table_foreach (hash, foreach_value_in_list, &data);
 }
 
+/**
+ * soup_message_set_auth:
+ * @msg: a #SoupMessage
+ * @auth: a #SoupAuth, or %NULL
+ *
+ * Sets @msg to authenticate to its destination using @auth, which
+ * must have already been fully authenticated. If @auth is %NULL, @msg
+ * will not authenticate to its destination.
+ **/
+void
+soup_message_set_auth (SoupMessage *msg, SoupAuth *auth)
+{
+	SoupMessagePrivate *priv;
+	char *token;
+
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	g_return_if_fail (auth == NULL || SOUP_IS_AUTH (auth));
+	g_return_if_fail (auth == NULL || soup_auth_is_authenticated (auth));
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	if (priv->auth) {
+		g_object_unref (priv->auth);
+		soup_message_remove_header (msg->request_headers, "Authorization");
+	}
+	priv->auth = auth;
+	if (!priv->auth)
+		return;
+
+	g_object_ref (priv->auth);
+	token = soup_auth_get_authorization (auth, msg);
+	soup_message_add_header (msg->request_headers, "Authorization", token);
+	g_free (token);
+}
+
+/**
+ * soup_message_get_auth:
+ * @msg: a #SoupMessage
+ *
+ * Gets the #SoupAuth used by @msg for authentication.
+ *
+ * Return value: the #SoupAuth used by @msg for authentication, or
+ * %NULL if @msg is unauthenticated.
+ **/
+SoupAuth *
+soup_message_get_auth (SoupMessage *msg)
+{
+	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
+
+	return SOUP_MESSAGE_GET_PRIVATE (msg)->auth;
+}
+
+/**
+ * soup_message_set_proxy_auth:
+ * @msg: a #SoupMessage
+ * @auth: a #SoupAuth, or %NULL
+ *
+ * Sets @msg to authenticate to its proxy using @auth, which must have
+ * already been fully authenticated. If @auth is %NULL, @msg will not
+ * authenticate to its proxy.
+ **/
+void
+soup_message_set_proxy_auth (SoupMessage *msg, SoupAuth *auth)
+{
+	SoupMessagePrivate *priv;
+	char *token;
+
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+	g_return_if_fail (auth == NULL || SOUP_IS_AUTH (auth));
+	g_return_if_fail (auth == NULL || soup_auth_is_authenticated (auth));
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	if (priv->proxy_auth) {
+		g_object_unref (priv->proxy_auth);
+		soup_message_remove_header (msg->request_headers,
+					    "Proxy-Authorization");
+	}
+	priv->proxy_auth = auth;
+	if (!priv->proxy_auth)
+		return;
+
+	g_object_ref (priv->proxy_auth);
+	token = soup_auth_get_authorization (auth, msg);
+	soup_message_add_header (msg->request_headers,
+				 "Proxy-Authorization", token);
+	g_free (token);
+}
+
+/**
+ * soup_message_get_proxy_auth:
+ * @msg: a #SoupMessage
+ *
+ * Gets the #SoupAuth used by @msg for authentication to its proxy..
+ *
+ * Return value: the #SoupAuth used by @msg for authentication to its
+ * proxy, or %NULL if @msg isn't authenticated to its proxy.
+ **/
+SoupAuth *
+soup_message_get_proxy_auth (SoupMessage *msg)
+{
+	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
+
+	return SOUP_MESSAGE_GET_PRIVATE (msg)->proxy_auth;
+}
+
 /**
  * soup_message_cleanup_response:
  * @req: a #SoupMessage
@@ -871,8 +981,9 @@ soup_message_is_keepalive (SoupMessage *msg)
  * @msg: a #SoupMessage
  * @uri: the new #SoupUri
  *
- * Changes the URI that @msg is directed to (generally as a result
- * of a redirect).
+ * Sets @msg's URI to @uri. If @msg has already been sent and you want
+ * to re-send it with the new URI, you need to call
+ * soup_session_requeue_message().
  **/
 void
 soup_message_set_uri (SoupMessage *msg, const SoupUri *uri)
@@ -882,12 +993,6 @@ soup_message_set_uri (SoupMessage *msg, const SoupUri *uri)
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 
-	if (priv->uri && uri) {
-		if (strcmp (priv->uri->host, uri->host) != 0)
-			soup_message_io_stop (msg);
-	} else if (!uri)
-		soup_message_io_stop (msg);
-
 	if (priv->uri)
 		soup_uri_free (priv->uri);
 	priv->uri = soup_uri_copy (uri);
@@ -989,9 +1094,7 @@ soup_message_get_response_encoding (SoupMessage *msg, guint *content_length)
 {
 	SoupMethodId method = soup_method_get_id (msg->method);
 
-	/* FIXME: should CONNECT really be here? Where does it say that? */
 	if (method == SOUP_METHOD_ID_HEAD ||
-	    method == SOUP_METHOD_ID_CONNECT ||
 	    msg->status_code  == SOUP_STATUS_NO_CONTENT ||
 	    msg->status_code  == SOUP_STATUS_NOT_MODIFIED ||
 	    SOUP_STATUS_IS_INFORMATIONAL (msg->status_code))
@@ -1027,7 +1130,9 @@ soup_message_get_response_encoding (SoupMessage *msg, guint *content_length)
 					*content_length = lval;
 				return SOUP_TRANSFER_CONTENT_LENGTH;
 			}
-		} else
+		} else if (method == SOUP_METHOD_ID_CONNECT)
+			return SOUP_TRANSFER_NONE;
+		else
 			return SOUP_TRANSFER_EOF;
 	}
 }
@@ -1110,7 +1215,7 @@ soup_message_add_chunk (SoupMessage   *msg,
 	chunk->length = length;
 
 	if (priv->chunks) {
-		g_slist_append (priv->last_chunk, chunk);
+		priv->last_chunk = g_slist_append (priv->last_chunk, chunk);
 		priv->last_chunk = priv->last_chunk->next;
 	} else {
 		priv->chunks = priv->last_chunk =
